/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
#include "lcd.h"
#include "usart.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */

void InitApp(void)
{
    /* Setup analog functionality and port direction */
	ANSELA = 0;
	ANSELB = 0;

	OPTION_REGbits.nRBPU = 0;	// Enable pullups
	WPUB = 0x0F;				// Enable pullups only for the buttons

	X = 1;
	Y = 1;

	TRISA &= ~(0x3F);			//LCD CONTROL
	TRISCbits.TRISC2 = 0;		//BLACKLIGHT

	PORTCbits.RC4 = 0;
	PORTCbits.RC3 = 0;
	TRISCbits.TRISC4 = 1;		// I2C SDA line
	TRISCbits.TRISC3 = 1;		// I2C SCK line

	TRISCbits.TRISC6 = 1;		// RX
	TRISCbits.TRISC7 = 1;		// TX

	

	
    /* Initialize peripherals */

		LCD_Init();
	I2C_Init();

	/* Init PWM */
	TRISCbits.TRISC2 = 1;
	PR2 = 0b11111111;
	T2CON = 0b0111111;
	CCPR1L = BLACKLIGHT;
	CCP1CON = 0b00111100;
	


    /* Enable interrupts */
	PIR1bits.TMR2IF = 0;
	PIE1bits.TMR2IE = 1;
	T2CONbits.TMR2ON = 1;
	
	PIR1bits.SSPIF = 0;
	PIE1bits.SSPIE = 1;
	
	INTCONbits.PEIE = 1;
	INTCONbits.GIE = 1;
}

void I2C_Init(void){
	SSPBUF = 0x00;
	SSPCON = 0x3E;
	SSPADD = 0x30<<1;
	SSPSTAT = 0x80;
}

