/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>

#include "user.h"        /* For true/false definition */
#include "lcd.h"
#include "gpio.h"
#include "usart.h"

/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/

/* Baseline devices don't have interrupts. Note that some PIC16's 
 * are baseline devices.  Unfortunately the baseline detection macro is 
 * _PIC12 */
#ifndef _PIC12

void interrupt isr(void) {
	if (PIR1bits.SSPIF) {
		PIR1bits.SSPIF = 0; // Clear interrupt flag
		if (SSPSTAT & 0x10) {

			switch (DATA_BUFFER[0]) {
			case GET_ID:
				DATA_OUT = BOARD_ID;
				break;

			case SET_TRIS:
				gpioSetTris(DATA_BUFFER[1], DATA_BUFFER[2]);
				break;

			case SET_LAT:
				gpioSetLat(DATA_BUFFER[1], DATA_BUFFER[2]);
				break;

			case GET_PORT:
				DATA_OUT = gpioGetPort(DATA_BUFFER[1]);
				break;

			case GET_BUT:
				DATA_OUT = gpioGetButtons();
				break;

			case LCD_CLR:
				LCD_Clear();
				break;

			case LCD_WR:
				LCD_Send_Symbol(DATA_BUFFER[1], DATA_BUFFER[2], DATA_BUFFER[3]);
				break;

			case LCD_BL:
				LCD_Set_BlackLight(DATA_BUFFER[1]);
				break;

			case GET_FRM:
				DATA_OUT = FRMWR_ID;
				break;

/**
 * COMMENT THIS
 */
//			case UART_EN:
//				if (DATA_BUFFER[1] == 0x00)
//					USART_Disable();
//				else
//					USART_Init();
//				break;
			default:
				break;
			}
		} else {
			uint8_t status = SSPSTAT & 0x2D;
			switch (status) {
			case 0x09:
				/* STATE 1 */
				FOOBAR = SSPBUF;
				DATA_INDEX = 0;
				SSPCONbits.CKP = 1;
				break;

			case 0x29:
				/* STATE 2 */
				DATA_BUFFER[DATA_INDEX++] = SSPBUF;

				if (DATA_INDEX == BUFFER_SIZE)
					DATA_INDEX = 0;
				SSPCONbits.CKP = 1;
				break;

			case 0x0C:
				/* STATE 3 */
				FOOBAR = SSPBUF;
				while (SSPSTATbits.BF);
				SSPBUF = DATA_OUT;
				SSPCONbits.CKP = 1;
				break;

			case 0x2C:
				/* STATE 4 */
				FOOBAR = SSPBUF;
				while (SSPSTATbits.BF);
				SSPBUF = 0x00;
				SSPCONbits.CKP = 1;
				break;

			case 0x28:
				break;

			default:
				break;
			}
		}


	} else if (PIR1bits.TMR2IF) {
		TRISCbits.TRISC2 = 0;
		CCPR1L = BLACKLIGHT;
		PIR1bits.TMR2IF = 0;
	}
	/**
	 * ALSO COMMENT THIS WHOLE SECTION
     */
#if 0
#ifndef __DEBUG
	else if (PIR1bits.RCIF) {
		if (RCREG != '\r') {
			if (RCREG != '\n')
				UART_BUFFER[UART_INDEX++] = RCREG;
		} else {
			UART_BUFFER[UART_INDEX] = 0;
			UART_INDEX = 0;
			char *pch;
			char uart_words[3][10];
			uint8_t i = 0;


			/* Parse the recieved string */
			pch = strtok(UART_BUFFER, ":");
			while (pch != NULL) {
				strcpy(uart_words[i++], pch);
				pch = strtok(NULL, ":");
			}

			if (i == 3) {
				if (!strcmp(uart_words[0], "TRIS")) {

					/* Get pin number*/
					if (uart_words[1][0] > 0x30 && uart_words[1][0] < 0x3A) {

						/* Get direction */
						if (uart_words[2][0] == '1' || uart_words[2][0] == '0') {
							gpioSetTris(uart_words[1][0] - 0x30, uart_words[2][0] - 0x30);
						}
					}
				} else if (!strcmp(uart_words[0], "LAT")) {

					/* Get pin number*/
					if (uart_words[1][0] > 0x30 && uart_words[1][0] < 0x3A) {

						/* Get direction */
						if (uart_words[2][0] == '1' || uart_words[2][0] == '0') {
							gpioSetLat(uart_words[1][0] - 0x30, uart_words[2][0] - 0x30);
						}
					}
				} else if (!strcmp(uart_words[0], "GOTOXY")) {
					X = strtol(uart_words[1], NULL, 0);
					if (X < 1 || X > 16)
						X = 1;

					Y = strtol(uart_words[2], NULL, 0);
					if (Y < 1 || Y > 2)
						Y = 1;


				}

			} else if (i == 2) {
				if (!strcmp(uart_words[0], "PORT")) {

					/* Get pin number*/
					if (uart_words[1][0] > 0x30 && uart_words[1][0] < 0x3A) {
						while (!TXSTAbits.TRMT);
						TXREG = gpioGetPort(uart_words[1][0] - 0x30) + 0x30;
					}

				} else if (!strcmp(uart_words[0], "BLKL")) {
					int blk = 0;
					blk = strtol(uart_words[1], NULL, 0);
					if (blk < 0 || blk > 255)
						return;
					else
						BLACKLIGHT = blk;


				} else if (!strcmp(uart_words[0], "WR")) {
					char *p;
					uint8_t x, y;
					p = uart_words[1];
					x = X;
					y = Y;
					while (*p) {
						LCD_Send_Symbol(y, x - 1, *p);
						p++;
						x++;
						if (x > 16) {
							x = 1;
							y++;
							if (y > 2)
								break;
						}

					}
				}
			} else if (i == 1) {
				if (!strcmp(uart_words[0], "BUT")) {
					while (!TXSTAbits.TRMT);
					TXREG = gpioGetButtons();


				} else if (!strcmp(uart_words[0], "CLEAR")) {
					LCD_Clear();


				}
			}

		}
		PIR1bits.RCIF = 0;
	}
#endif

#endif
}
#endif


